/*
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

import QtQuick 2.12
import QtQuick.Layouts 1.12
import QtGraphicalEffects 1.12
import org.ukui.sidebar.ui 1.0
import org.ukui.sidebar.core 1.0
import org.ukui.sidebar.items 1.0
import org.ukui.sidebar.shortcut.core 1.0

Item {
    id: root
    property int duration: 300

    Component.onCompleted: {
        sidebarWindow.requestShowContent.connect(showContent);
        sidebarWindow.requestHideContent.connect(hideContent);

        handGestureHelper.controlCenterCalled.connect(trackMouse)
        handGestureHelper.right2LeftReleased.connect(mouseReleased)
    }

    function updateProperty(active) {
        if (active) {
            shortcutArea.x = shortcutAreaBase.width + sidebarWindow.windowPadding;
            windowBlurHelper.radius = 0;

        } else {
            shortcutArea.x = 0;
        }

        windowBlurHelper.enable = sidebarWindow.isTabletMode;
    }

    function isRunning(active) {
        return shortcutAnimation.running;
    }

    function showContent() {
        shortcutAreaBase.startAnimation(shortcutAreaBase.width + sidebarWindow.windowPadding, 0, root.duration, true);
    }

    function hideContent() {
        shortcutAreaBase.startAnimation(0, shortcutAreaBase.width + sidebarWindow.windowPadding, root.duration, false);
    }

    function trackMouse(posX) {
        if (sidebarWindow.contentVisible) {
            return;
        }

        if (shortcutAnimation.running === true) {
            shortcutAnimation.running = false;
        }

        if (!sidebarWindow.visible) {
            updateProperty(true);
            sidebarWindow.show();
        }

        // 鼠标位置与当前主屏幕右边界的距离
        let distanceX = sidebarWindow.primaryScreenRight - posX;
        if (distanceX < 0) {
            if (sidebarWindow.visible) {
                sidebarWindow.hide();
            }
            return;
        }

        // 加上padding忽略窗口的边距
        let shortcutWidth = layoutConfig.width(LayoutComponent.SidebarMain) + sidebarWindow.windowPadding;
        if (distanceX > shortcutWidth) {
            distanceX = shortcutWidth;
        }

        shortcutArea.x = shortcutWidth - distanceX;
    }

    // 唤起侧边栏手势释放事件
    function mouseReleased(posX) {
        if (!sidebarWindow.visible || sidebarWindow.contentVisible) {
            return;
        }

        // 鼠标位置与当前主屏幕右边界的距离
        let distanceX = sidebarWindow.primaryScreenRight - posX;
        let shortcutWidth = layoutConfig.width(LayoutComponent.SidebarMain) + sidebarWindow.windowPadding;
        let duration = 50;

        // 如果为负值，快速隐藏
        if (distanceX < 0) {
            shortcutAreaBase.startAnimation(shortcutArea.x, shortcutWidth, duration, false);
            return
        }

        // 屏幕宽度的5%
        if (distanceX <= sidebarWindow.minimumThreshold) {
            duration = Math.floor(root.duration * (distanceX / shortcutWidth));
            shortcutAreaBase.startAnimation(shortcutArea.x, shortcutWidth, duration, false);
            return;
        }

        duration = Math.floor(root.duration * (shortcutArea.x / shortcutWidth));
        shortcutAreaBase.startAnimation(shortcutArea.x, 0, duration, true);
    }

    MouseArea {
        anchors.fill: parent
        property int startPosX: 0
        property int shortcutAreaStartPosX: 0
        property int shortcutAreaWidth: 0

        onPressed: function(mouse) {
            // pc模式下拒绝接受事件
            if (!sidebarWindow.isTabletMode) {
                mouse.accepted = false;
                return;
            }
            startPosX = mouse.x;
            shortcutAreaStartPosX = shortcutArea.x;
            shortcutAreaWidth = layoutConfig.width(LayoutComponent.SidebarMain) + sidebarWindow.windowPadding;
        }

        onReleased: function(mouse) {
            // 模仿点击事件
            if (Math.abs(mouse.x - startPosX) < 2) {
                if ((mouse.x > shortcutAreaBase.x) && (mouse.x < (shortcutAreaBase.x + shortcutAreaBase.width))
                    && (mouse.y > (shortcutAreaBase.height - shortcutArea.getShortcutAreaHeight())) && (mouse.y < shortcutAreaBase.height)) {
                    return;
                }

                shortcutAreaBase.startAnimation(shortcutArea.x, shortcutAreaBase.width + sidebarWindow.windowPadding, root.duration, false);
                return;
            }

            // x 距离屏幕右侧的距离，0 <= shortcutArea.x <= shortcutAreaWidth
            let duration = root.duration;

            // 移动超过5%时，隐藏侧边栏,否则继续显示
            if (shortcutArea.x > sidebarWindow.minimumThreshold) {
                let distanceX = shortcutAreaWidth - shortcutArea.x;
                duration = Math.floor(root.duration * (distanceX / shortcutAreaWidth));
                shortcutAreaBase.startAnimation(shortcutArea.x, shortcutAreaBase.width + sidebarWindow.windowPadding, duration, false);
                return;
            }

            duration = Math.floor(root.duration * (shortcutArea.x / shortcutAreaWidth));
            shortcutAreaBase.startAnimation(shortcutArea.x, 0, duration, true);
        }

        onPositionChanged: function(mouse) {
            if (pressed) {
                let distanceX = mouse.x - startPosX;
                let newX = shortcutAreaStartPosX + distanceX;

                // 边界值判断,确保：0 <= shortcutArea.x <= shortcutAreaWidth
                if (newX < 0) {
                    newX = 0;

                } else if (newX > shortcutAreaWidth) {
                    newX = shortcutAreaWidth;
                }

                shortcutArea.x = newX;
            }
        }
    }

    NumberAnimation {
        id: showGroundGlass
        target: windowBlurHelper
        property: "radius"
        duration: root.duration
        from: 0
        to: 4000
    }

    // 阴影区域
    ShadowComponent {
        id: shadowArea;
        z: 10;
        visible: !sidebarWindow.isTabletMode && isOpenGLEnv;
        radius: sidebarWindow.radius;
        x: shortcutAreaBase.x - sidebarWindow.windowPadding;
        y: shortcutAreaBase.y - sidebarWindow.windowPadding;
        width: shortcutAreaBase.width + sidebarWindow.windowPadding*2;
        height: shortcutAreaBase.height + sidebarWindow.windowPadding*2;
    }

    Rectangle {
        id: desktopBac;
        z: 0;
        anchors.fill: parent;
        visible: sidebarWindow.isTabletMode;
        onOpacityChanged: {
            update();
        }

        function updateColor() {
            color = colorHelper.mask();
        }
        Component.onCompleted: {
            updateColor();
            colorHelper.styleColorChanged.connect(updateColor);
        }

        Component.onDestruction: {
            colorHelper.styleColorChanged.disconnect(updateColor);
        }
    }

    Item {
        id: shortcutAreaBase
        anchors.margins: sidebarWindow.windowPadding
        z: 100

        states: [
            State {
                when: !sidebarWindow.isTabletMode
                AnchorChanges {
                    target: shortcutAreaBase
                    anchors.left: root.left
                    anchors.top: root.top
                    anchors.right: root.right
                    anchors.bottom: root.bottom
                }
            },
            State {
                when: sidebarWindow.isTabletMode
                AnchorChanges {
                    target: shortcutAreaBase
                    anchors.left: undefined
                    anchors.top: root.top
                    anchors.right: root.right
                    anchors.bottom: root.bottom
                }
                PropertyChanges {
                    target: shortcutAreaBase
                    width: layoutConfig.width(LayoutComponent.SidebarMain)
                }
            }
        ]

        function startAnimation(from, to, duration, isShow) {
            // console.log("=== start animation ===", from, to, duration, isShow)
            if (shortcutAnimation.running) {
                return;
            }
            shortcutAnimation.from = from;
            shortcutAnimation.to = to;
            shortcutAnimation.duration = duration < 50 ? 50 : duration;
            shortcutAnimation.isShow = isShow;

            shortcutAnimation.start();
        }

        NumberAnimation {
            id: shortcutAnimation
            property bool isShow: false

            target: shortcutArea
            property: "x"
            easing.type: Easing.InOutQuad

            onStopped: {
                if (isShow) {
                    sidebarWindow.contentVisible = true;
                    if (!sidebarWindow.isTabletMode) {
                        windowBlurHelper.enable = true;
                        showGroundGlass.start();
                    }
                } else {
                    sidebarWindow.contentVisible = false;
                    sidebarWindow.hide();
                }
            }
        }

        ShortcutArea {
            id: shortcutArea;
            visible: true
            width: shortcutLayoutHelper.width;
            height: parent.height;
            clip: true;
            layer.enabled: isOpenGLEnv;
            layer.effect: OpacityMask {
                maskSource: Rectangle {
                    width: shortcutArea.width;
                    height: shortcutArea.height;
                    radius: sidebarWindow.radius;
                }
            }

            onXChanged: {
                shadowArea.x = shortcutAreaBase.x - shadowArea.shadowWidth + shortcutArea.x;

                if (sidebarWindow.isTabletMode) {
                    let shortAreaWidth = width + sidebarWindow.windowPadding;
                    let rightWidth = Math.abs(Math.floor(shortAreaWidth - x));
                    windowBlurHelper.radius = Math.floor(4000 * (rightWidth / shortAreaWidth));
                    windowBlurHelper.enable = (rightWidth !== 0);
                    desktopBac.opacity = Math.abs(rightWidth / shortAreaWidth);
                }
            }
        }
    }
}
